import pytest
from rubrik_cdm.exceptions import CDMVersionException

import urllib3
urllib3.disable_warnings()


@pytest.mark.integration
def test_cluster_version_check(rubrik, monkeypatch):

    with pytest.raises(CDMVersionException):
        rubrik.cluster_version_check("5.0")



import pytest
from rubrik_cdm.exceptions import InvalidParameterException, CDMVersionException, InvalidTypeException
from rubrik_cdm import Connect


def test_add_physical_host_empty_hostname_list(rubrik):

    with pytest.raises(InvalidParameterException) as error:
        rubrik.add_physical_host([])

    error_message = error.value.args[0]

    assert error_message == "The provided hostname list is empty."


def test_add_physical_host_idempotence(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    assert rubrik.add_physical_host("hostname") == \
        "No change required. The host 'hostname' is already connected to the Rubrik cluster."


def test_add_physical_host_list_idempotence(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname1",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                },
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname2",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    assert rubrik.add_physical_host(["hostname1", "hostname2"]) == \
        "No Change Required. All Hosts Already added or supplied list was empty"


def test_add_physical_host(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "string",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 1
        }

    def mock_post_v1_host():
        return {
            "id": "string",
            "name": "string",
            "hostname": "string",
            "primaryClusterId": "string",
            "operatingSystem": "string",
            "operatingSystemType": "string",
            "status": "string",
            "nasBaseConfig": {
                "vendorType": "string",
                "apiUsername": "string",
                "apiCertificate": "string",
                "apiHostname": "string",
                "apiEndpoint": "string",
                "zoneName": "string"
            },
            "mssqlCbtEnabled": "Enabled",
            "mssqlCbtEffectiveStatus": "On",
            "organizationId": "string",
            "organizationName": "string",
            "agentId": "string",
            "compressionEnabled": True,
            "isRelic": True,
            "mssqlCbtDriverInstalled": True,
            "hostVfdEnabled": "Enabled",
            "hostVfdDriverState": "NotInstalled",
            "oracleSysDbaUser": "string"
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
    mock_post.return_value = mock_post_v1_host()

    assert rubrik.add_physical_host("hostname") == mock_post_v1_host()


def test_add_physical_host_list(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "string",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                },
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "string",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_post_v1_host():
        return {
            "id": "string",
            "name": "string",
            "hostname": "string",
            "primaryClusterId": "string",
            "operatingSystem": "string",
            "operatingSystemType": "string",
            "status": "string",
            "nasBaseConfig": {
                "vendorType": "string",
                "apiUsername": "string",
                "apiCertificate": "string",
                "apiHostname": "string",
                "apiEndpoint": "string",
                "zoneName": "string"
            },
            "mssqlCbtEnabled": "Enabled",
            "mssqlCbtEffectiveStatus": "On",
            "organizationId": "string",
            "organizationName": "string",
            "agentId": "string",
            "compressionEnabled": True,
            "isRelic": True,
            "mssqlCbtDriverInstalled": True,
            "hostVfdEnabled": "Enabled",
            "hostVfdDriverState": "NotInstalled",
            "oracleSysDbaUser": "string"
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
    mock_post.return_value = mock_post_v1_host()

    assert rubrik.add_physical_host("hostname") == mock_post_v1_host()


def test_delete_physical_host_idempotence(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "string",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    assert rubrik.delete_physical_host("hostname") == \
        "No change required. The host 'hostname' is not connected to the Rubrik cluster."


def test_delete_physical_host(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                },
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "string",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_delete_v1_host_id():
        return {"status_code: 204"}

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    mock_delete = mocker.patch('rubrik_cdm.Connect.delete', autospec=True, spec_set=True)
    mock_delete.return_value = mock_delete_v1_host_id()

    assert rubrik.delete_physical_host("hostname") == mock_delete_v1_host_id()


def test_create_physical_fileset_invalid_operating_system(rubrik):

    with pytest.raises(InvalidParameterException) as error:
        rubrik.create_physical_fileset("name", "not_a_valid_operating_system",
                                       "include", "exclude", "exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The create_physical_fileset() operating_system argument must be one of the following: ['Linux', 'Windows']."


def test_create_physical_fileset_invalid_follow_network_shares(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_physical_fileset("name", "Linux",
                                       "include", "exclude", "exclude_exception", follow_network_shares="not_a_valid_follow_network_shares")

    error_message = error.value.args[0]

    assert error_message == "The 'follow_network_shares' argument must be True or False."


def test_create_physical_fileset_invalid_backup_hidden_folders(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_physical_fileset("name", "Linux",
                                       "include", "exclude", "exclude_exception", backup_hidden_folders="not_a_valid_backup_hidden_folders")

    error_message = error.value.args[0]

    assert error_message == "The 'backup_hidden_folders' argument must be True or False."


def test_create_physical_fileset_invalid_include(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_physical_fileset("name", "Linux", "not_a_valid_include", "exclude", "exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'include' argument must be a list object."


def test_create_physical_fileset_invalid_exclude(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_physical_fileset("name", "Linux", [""], "not_a_valid_exclude", "exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'exclude' argument must be a list object."


def test_create_physical_fileset_invalid_exclude_exception(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_physical_fileset("name", "Linux", [""], [""], "not_a_valid_exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'exclude_exception' argument must be a list object."


def test_create_physical_fileset_idempotence(rubrik, mocker):

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "name",
                    "includes": [
                        "includes"
                    ],
                    "excludes": [
                        "excludes"
                    ],
                    "exceptions": [
                        "exceptions"
                    ],
                    "operatingSystemType": "Linux",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_fileset_template()

    assert rubrik.create_physical_fileset("name", "Linux", ["includes"], ["excludes"], ["exceptions"], True, True) == \
        "No change required. The Rubrik cluster already has a Linux Fileset named 'name' configured with the provided variables."


def test_create_physical_fileset(rubrik, mocker):

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [],
            "total": 1
        }

    def mock_post_v1_fileset_template_bulk():
        return {
            "allowBackupNetworkMounts": True,
            "allowBackupHiddenFoldersInNetworkMounts": True,
            "useWindowsVss": True,
            "name": "string",
            "includes": [
                "string"
            ],
            "excludes": [
                "string"
            ],
            "exceptions": [
                "string"
            ],
            "operatingSystemType": "UnixLike",
            "shareType": "NFS",
            "preBackupScript": "string",
            "postBackupScript": "string",
            "backupScriptTimeout": 0,
            "backupScriptErrorHandling": "string",
            "isArrayEnabled": True,
            "id": "string",
            "primaryClusterId": "string",
            "isArchived": True,
            "hostCount": 0,
            "shareCount": 0
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_fileset_template()

    mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
    mock_post.return_value = mock_post_v1_fileset_template_bulk()

    assert rubrik.create_physical_fileset("name", "Linux", ["includes"], ["excludes"], ["exceptions"], True, True) == \
        mock_post_v1_fileset_template_bulk()


def test_create_nas_fileset_invalid_share_type(rubrik):

    with pytest.raises(InvalidParameterException) as error:
        rubrik.create_nas_fileset("name", "not_a_valid_share_type", "include", "exclude", "exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The create_fileset() share_type argument must be one of the following: ['NFS', 'SMB']."


def test_create_nas_fileset_invalid_follow_network_shares(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_nas_fileset(
            "name",
            "NFS",
            "include",
            "exclude",
            "exclude_exception",
            "not_a_valid_follow_network_share")

    error_message = error.value.args[0]

    assert error_message == "The 'follow_network_shares' argument must be True or False."


def test_create_nas_fileset_invalid_include(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_nas_fileset(
            "name", "NFS", "not_a_valid_include", "exclude", "exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'include' argument must be a list object."


def test_create_nas_fileset_invalid_exclude(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_nas_fileset(
            "name", "NFS", [""], "not_a_valid_exclude", "exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'exclude' argument must be a list object."


def test_create_nas_fileset_invalid_exclude_exception(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.create_nas_fileset(
            "name", "NFS", [""], [""], "not_a_valid_exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'exclude_exception' argument must be a list object."


def test_create_nas_fileset_idempotence(rubrik, mocker):

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "name",
                    "includes": [
                        "includes"
                    ],
                    "excludes": [
                        "excludes"
                    ],
                    "exceptions": [
                        "exceptions"
                    ],
                    "operatingSystemType": "Linux",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_fileset_template()

    assert rubrik.create_nas_fileset("name", "NFS", ["includes"], ["excludes"], ["exceptions"], True) == \
        "No change required. The Rubrik cluster already has a NAS Fileset named 'name' configured with the provided variables."


@pytest.mark.parametrize('share', ["NFS", "SMB"])
def test_create_nas_fileset(rubrik, mocker, share):

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [],
            "total": 1
        }

    def mock_post_v1_fileset_template_bulk():
        return {
            "allowBackupNetworkMounts": True,
            "allowBackupHiddenFoldersInNetworkMounts": True,
            "useWindowsVss": True,
            "name": "string",
            "includes": [
                "string"
            ],
            "excludes": [
                "string"
            ],
            "exceptions": [
                "string"
            ],
            "operatingSystemType": "UnixLike",
            "shareType": "NFS",
            "preBackupScript": "string",
            "postBackupScript": "string",
            "backupScriptTimeout": 0,
            "backupScriptErrorHandling": "string",
            "isArrayEnabled": True,
            "id": "string",
            "primaryClusterId": "string",
            "isArchived": True,
            "hostCount": 0,
            "shareCount": 0
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_fileset_template()

    mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
    mock_post.return_value = mock_post_v1_fileset_template_bulk()

    assert rubrik.create_nas_fileset("name", share, ["includes"], ["excludes"], ["exceptions"], True) == \
        mock_post_v1_fileset_template_bulk()


def test_assign_physical_host_fileset_invalid_operating_system(rubrik):

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "not_a_valid_operting_system", "sla_name")

    error_message = error.value.args[0]

    assert error_message == "The assign_physical_host_fileset() operating_system argument must be one of the following: ['Linux', 'Windows']."


def test_assign_physical_host_fileset_invalid_follow_network_shares(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.assign_physical_host_fileset(
            "hostname",
            "fileset_name",
            "Linux",
            "sla_name",
            follow_network_shares="not_a_valid_follow_network_shares")

    error_message = error.value.args[0]

    assert error_message == "The 'follow_network_shares' argument must be True or False."


def test_assign_physical_host_fileset_invalid_backup_hidden_folders(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.assign_physical_host_fileset(
            "hostname",
            "fileset_name",
            "Linux",
            "sla_name",
            backup_hidden_folders="not_a_valid_backup_hidden_folders")

    error_message = error.value.args[0]

    assert error_message == "The 'backup_hidden_folders' argument must be True or False."


def test_assign_physical_host_fileset_invalid_include(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.assign_physical_host_fileset(
            "hostname",
            "fileset_name",
            "Linux",
            "sla_name",
            include="not_a_valid_include")

    error_message = error.value.args[0]

    assert error_message == "The 'include' argument must be a list object."


def test_assign_physical_host_fileset_invalid_exclude(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.assign_physical_host_fileset(
            "hostname",
            "fileset_name",
            "Linux",
            "sla_name",
            exclude="not_a_valid_exclude")

    error_message = error.value.args[0]

    assert error_message == "The 'exclude' argument must be a list object."


def test_assign_physical_host_fileset_invalid_exclude_exception(rubrik):

    with pytest.raises(InvalidTypeException) as error:
        rubrik.assign_physical_host_fileset(
            "hostname",
            "fileset_name",
            "Linux",
            "sla_name",
            exclude_exception="not_a_valid_exclude_exception")

    error_message = error.value.args[0]

    assert error_message == "The 'exclude_exception' argument must be a list object."


def test_assign_physical_host_fileset_invalid_hostname(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [],
            "total": 0
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name")

    error_message = error.value.args[0]

    assert error_message == "The Rubrik cluster is not connected to a Linux physical host named 'hostname'."


def test_assign_physical_host_fileset_invalid_hostname_no_match(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [],
            "total": 0
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name")

    error_message = error.value.args[0]

    assert error_message == "The Rubrik cluster is not connected to a Linux physical host named 'hostname'."


def test_assign_physical_host_fileset_invalid_hostname_close_match(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "string",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.return_value = mock_get_v1_host()

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name")

    error_message = error.value.args[0]

    assert error_message == "The Rubrik cluster is not connected to a Linux physical host named 'hostname'."


def test_assign_physical_host_fileset_invalid_fileset_name(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 1
        }

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [],
            "total": 0
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.side_effect = [mock_get_v1_host(), mock_get_v1_fileset_template()]

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name")

    error_message = error.value.args[0]

    assert error_message == "The Rubrik cluster does not have a Linux Fileset named 'fileset_name'."


def test_assign_physical_host_fileset_invalid_fileset_name_multiple_matches_not_specific(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                },
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 2
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.side_effect = [mock_get_v1_host(), mock_get_v1_fileset_template()]

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name")

    error_message = error.value.args[0]

    assert error_message == "The Rubrik cluster contains multiple Linux Filesets named 'fileset_name'. Please populate all function arguments to find a more specific match."


def test_assign_physical_host_fileset_invalid_fileset_name_multiple_matches_specific(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                },
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 2
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.side_effect = [mock_get_v1_host(), mock_get_v1_fileset_template()]

    with pytest.raises(InvalidParameterException) as error:
        rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name", [""], ["", True, True])

    error_message = error.value.args[0]

    assert error_message == "The Rubrik cluster contains multiple Linux Filesets named 'fileset_name' that match all of the populate function arguments. Please use a unique Fileset."


def test_assign_physical_host_fileset_patch_sla(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 1
        }

    def mock_get_v1_sla_domain():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string_sla_id",
                    "primaryClusterId": "string",
                    "name": "sla_name",
                    "frequencies": [
                        {
                            "timeUnit": "string",
                            "frequency": 0,
                            "retention": 0
                        }
                    ],
                    "allowedBackupWindows": [
                        {
                            "startTimeAttributes": {
                                "minutes": 0,
                                "hour": 0,
                                "dayOfWeek": 0
                            },
                            "durationInHours": 0
                        }
                    ],
                    "firstFullAllowedBackupWindows": [
                        {
                            "startTimeAttributes": {
                                "minutes": 0,
                                "hour": 0,
                                "dayOfWeek": 0
                            },
                            "durationInHours": 0
                        }
                    ],
                    "localRetentionLimit": 0,
                    "maxLocalRetentionLimit": 0,
                    "archivalSpecs": [
                        {
                            "locationId": "string",
                            "archivalThreshold": 0
                        }
                    ],
                    "replicationSpecs": [
                        {
                            "locationId": "string",
                            "retentionLimit": 0
                        }
                    ],
                    "numDbs": 0,
                    "numOracleDbs": 0,
                    "numFilesets": 0,
                    "numHypervVms": 0,
                    "numNutanixVms": 0,
                    "numManagedVolumes": 0,
                    "numStorageArrayVolumeGroups": 0,
                    "numWindowsVolumeGroups": 0,
                    "numLinuxHosts": 0,
                    "numShares": 0,
                    "numWindowsHosts": 0,
                    "numVms": 0,
                    "numEc2Instances": 0,
                    "numVcdVapps": 0,
                    "numProtectedObjects": 0,
                    "isDefault": True,
                    "uiColor": "string"
                }
            ],
            "total": 1
        }

    def mock_get_v1_fileset():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "id": "string",
                    "name": "string",
                    "configuredSlaDomainId": "string",
                    "configuredSlaDomainName": "string",
                    "primaryClusterId": "string",
                    "hostId": "string",
                    "shareId": "string",
                    "hostName": "string",
                    "templateId": "string",
                    "templateName": "string",
                    "operatingSystemType": "string",
                    "effectiveSlaDomainId": "string",
                    "effectiveSlaDomainName": "string",
                    "effectiveSlaDomainPolarisManagedId": "string",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "isRelic": True,
                    "arraySpec": {
                        "proxyHostId": "string"
                    },
                    "isPassthrough": True
                }
            ],
            "total": 1
        }

    def mock_patch_v1_fileset():
        return {
            "configuredSlaDomainId": "string",
            "allowBackupNetworkMounts": True,
            "allowBackupHiddenFoldersInNetworkMounts": True,
            "useWindowsVss": True,
            "id": "string",
            "name": "string",
            "configuredSlaDomainName": "string",
            "primaryClusterId": "string",
            "hostId": "string",
            "shareId": "string",
            "hostName": "string",
            "templateId": "string",
            "templateName": "string",
            "operatingSystemType": "string",
            "effectiveSlaDomainId": "string",
            "effectiveSlaDomainName": "string",
            "effectiveSlaDomainPolarisManagedId": "string",
            "includes": [
                "string"
            ],
            "excludes": [
                "string"
            ],
            "exceptions": [
                "string"
            ],
            "isRelic": True,
            "arraySpec": {
                "proxyHostId": "string"
            },
            "isPassthrough": True,
            "protectionDate": "2019-05-23T15:36:06.889Z",
            "snapshotCount": 0,
            "archivedSnapshotCount": 0,
            "snapshots": [
                {
                    "id": "string",
                    "date": "2019-05-23T15:36:06.889Z",
                    "expirationDate": "2019-05-23T15:36:06.889Z",
                    "sourceObjectType": "string",
                    "isOnDemandSnapshot": True,
                    "cloudState": 0,
                    "consistencyLevel": "string",
                    "indexState": 0,
                    "replicationLocationIds": [
                        "string"
                    ],
                    "archivalLocationIds": [
                        "string"
                    ],
                    "slaId": "string",
                    "slaName": "string",
                    "filesetName": "string",
                    "fileCount": 0
                }
            ],
            "localStorage": 0,
            "archiveStorage": 0,
            "preBackupScript": "string",
            "postBackupScript": "string",
            "backupScriptTimeout": 0,
            "backupScriptErrorHandling": "string"
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.side_effect = [
        mock_get_v1_host(),
        mock_get_v1_fileset_template(),
        mock_get_v1_sla_domain(),
        mock_get_v1_fileset()]

    mock_patch = mocker.patch('rubrik_cdm.Connect.patch', autospec=True, spec_set=True)
    mock_patch.return_value = mock_patch_v1_fileset()

    assert rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name") == \
        mock_patch_v1_fileset()


def test_assign_physical_host_fileset_idempotence(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 1
        }

    def mock_get_v1_sla_domain():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string_sla_id",
                    "primaryClusterId": "string",
                    "name": "sla_name",
                    "frequencies": [
                        {
                            "timeUnit": "string",
                            "frequency": 0,
                            "retention": 0
                        }
                    ],
                    "allowedBackupWindows": [
                        {
                            "startTimeAttributes": {
                                "minutes": 0,
                                "hour": 0,
                                "dayOfWeek": 0
                            },
                            "durationInHours": 0
                        }
                    ],
                    "firstFullAllowedBackupWindows": [
                        {
                            "startTimeAttributes": {
                                "minutes": 0,
                                "hour": 0,
                                "dayOfWeek": 0
                            },
                            "durationInHours": 0
                        }
                    ],
                    "localRetentionLimit": 0,
                    "maxLocalRetentionLimit": 0,
                    "archivalSpecs": [
                        {
                            "locationId": "string",
                            "archivalThreshold": 0
                        }
                    ],
                    "replicationSpecs": [
                        {
                            "locationId": "string",
                            "retentionLimit": 0
                        }
                    ],
                    "numDbs": 0,
                    "numOracleDbs": 0,
                    "numFilesets": 0,
                    "numHypervVms": 0,
                    "numNutanixVms": 0,
                    "numManagedVolumes": 0,
                    "numStorageArrayVolumeGroups": 0,
                    "numWindowsVolumeGroups": 0,
                    "numLinuxHosts": 0,
                    "numShares": 0,
                    "numWindowsHosts": 0,
                    "numVms": 0,
                    "numEc2Instances": 0,
                    "numVcdVapps": 0,
                    "numProtectedObjects": 0,
                    "isDefault": True,
                    "uiColor": "string"
                }
            ],
            "total": 1
        }

    def mock_get_v1_fileset():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "id": "string",
                    "name": "string",
                    "configuredSlaDomainId": "string_sla_id",
                    "configuredSlaDomainName": "string",
                    "primaryClusterId": "string",
                    "hostId": "string",
                    "shareId": "string",
                    "hostName": "string",
                    "templateId": "string",
                    "templateName": "string",
                    "operatingSystemType": "string",
                    "effectiveSlaDomainId": "string",
                    "effectiveSlaDomainName": "string",
                    "effectiveSlaDomainPolarisManagedId": "string",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "isRelic": True,
                    "arraySpec": {
                        "proxyHostId": "string"
                    },
                    "isPassthrough": True
                }
            ],
            "total": 1
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.side_effect = [
        mock_get_v1_host(),
        mock_get_v1_fileset_template(),
        mock_get_v1_sla_domain(),
        mock_get_v1_fileset()]

    assert rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name") == \
        "No change required. The Linux Fileset 'fileset_name' is already assigned to the SLA Domain 'sla_name' on the physical host 'hostname'."


def test_assign_physical_host_fileset_no_current_fileset(rubrik, mocker):

    def mock_get_v1_host():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string",
                    "name": "string",
                    "hostname": "hostname",
                    "primaryClusterId": "string",
                    "operatingSystem": "string",
                    "operatingSystemType": "string",
                    "status": "string",
                    "nasBaseConfig": {
                        "vendorType": "string",
                        "apiUsername": "string",
                        "apiCertificate": "string",
                        "apiHostname": "string",
                        "apiEndpoint": "string",
                        "zoneName": "string"
                    },
                    "mssqlCbtEnabled": "Enabled",
                    "mssqlCbtEffectiveStatus": "On",
                    "organizationId": "string",
                    "organizationName": "string"
                }
            ],
            "total": 2
        }

    def mock_get_v1_fileset_template():
        return {
            "hasMore": True,
            "data": [
                {
                    "allowBackupNetworkMounts": True,
                    "allowBackupHiddenFoldersInNetworkMounts": True,
                    "useWindowsVss": True,
                    "name": "fileset_name",
                    "includes": [
                        "string"
                    ],
                    "excludes": [
                        "string"
                    ],
                    "exceptions": [
                        "string"
                    ],
                    "operatingSystemType": "UnixLike",
                    "shareType": "NFS",
                    "preBackupScript": "string",
                    "postBackupScript": "string",
                    "backupScriptTimeout": 0,
                    "backupScriptErrorHandling": "string",
                    "isArrayEnabled": True,
                    "id": "string",
                    "primaryClusterId": "string",
                    "isArchived": True,
                    "hostCount": 0,
                    "shareCount": 0
                }
            ],
            "total": 1
        }

    def mock_get_v1_sla_domain():
        return {
            "hasMore": True,
            "data": [
                {
                    "id": "string_sla_id",
                    "primaryClusterId": "string",
                    "name": "sla_name",
                    "frequencies": [
                        {
                            "timeUnit": "string",
                            "frequency": 0,
                            "retention": 0
                        }
                    ],
                    "allowedBackupWindows": [
                        {
                            "startTimeAttributes": {
                                "minutes": 0,
                                "hour": 0,
                                "dayOfWeek": 0
                            },
                            "durationInHours": 0
                        }
                    ],
                    "firstFullAllowedBackupWindows": [
                        {
                            "startTimeAttributes": {
                                "minutes": 0,
                                "hour": 0,
                                "dayOfWeek": 0
                            },
                            "durationInHours": 0
                        }
                    ],
                    "localRetentionLimit": 0,
                    "maxLocalRetentionLimit": 0,
                    "archivalSpecs": [
                        {
                            "locationId": "string",
                            "archivalThreshold": 0
                        }
                    ],
                    "replicationSpecs": [
                        {
                            "locationId": "string",
                            "retentionLimit": 0
                        }
                    ],
                    "numDbs": 0,
                    "numOracleDbs": 0,
                    "numFilesets": 0,
                    "numHypervVms": 0,
                    "numNutanixVms": 0,
                    "numManagedVolumes": 0,
                    "numStorageArrayVolumeGroups": 0,
                    "numWindowsVolumeGroups": 0,
                    "numLinuxHosts": 0,
                    "numShares": 0,
                    "numWindowsHosts": 0,
                    "numVms": 0,
                    "numEc2Instances": 0,
                    "numVcdVapps": 0,
                    "numProtectedObjects": 0,
                    "isDefault": True,
                    "uiColor": "string"
                }
            ],
            "total": 1
        }

    def mock_get_v1_fileset():
        return {
            "hasMore": True,
            "data": [],
            "total": 0
        }

    def mock_post_v1_fileset():
        return {
            "configuredSlaDomainId": "string",
            "allowBackupNetworkMounts": True,
            "allowBackupHiddenFoldersInNetworkMounts": True,
            "useWindowsVss": True,
            "id": "string",
            "name": "string",
            "configuredSlaDomainName": "string",
            "primaryClusterId": "string",
            "hostId": "string",
            "shareId": "string",
            "hostName": "string",
            "templateId": "string",
            "templateName": "string",
            "operatingSystemType": "string",
            "effectiveSlaDomainId": "string",
            "effectiveSlaDomainName": "string",
            "effectiveSlaDomainPolarisManagedId": "string",
            "includes": [
                "string"
            ],
            "excludes": [
                "string"
            ],
            "exceptions": [
                "string"
            ],
            "isRelic": True,
            "arraySpec": {
                "proxyHostId": "string"
            },
            "isPassthrough": True,
            "protectionDate": "2019-05-23T15:36:06.821Z",
            "snapshotCount": 0,
            "archivedSnapshotCount": 0,
            "snapshots": [
                {
                    "id": "string",
                    "date": "2019-05-23T15:36:06.821Z",
                    "expirationDate": "2019-05-23T15:36:06.821Z",
                    "sourceObjectType": "string",
                    "isOnDemandSnapshot": True,
                    "cloudState": 0,
                    "consistencyLevel": "string",
                    "indexState": 0,
                    "replicationLocationIds": [
                        "string"
                    ],
                    "archivalLocationIds": [
                        "string"
                    ],
                    "slaId": "string",
                    "slaName": "string",
                    "filesetName": "string",
                    "fileCount": 0
                }
            ],
            "localStorage": 0,
            "archiveStorage": 0,
            "preBackupScript": "string",
            "postBackupScript": "string",
            "backupScriptTimeout": 0,
            "backupScriptErrorHandling": "string"
        }

    def mock_patch_v1_fileset():
        return {
            "configuredSlaDomainId": "string",
            "allowBackupNetworkMounts": True,
            "allowBackupHiddenFoldersInNetworkMounts": True,
            "useWindowsVss": True,
            "id": "string",
            "name": "string",
            "configuredSlaDomainName": "string",
            "primaryClusterId": "string",
            "hostId": "string",
            "shareId": "string",
            "hostName": "string",
            "templateId": "string",
            "templateName": "string",
            "operatingSystemType": "string",
            "effectiveSlaDomainId": "string",
            "effectiveSlaDomainName": "string",
            "effectiveSlaDomainPolarisManagedId": "string",
            "includes": [
                "string"
            ],
            "excludes": [
                "string"
            ],
            "exceptions": [
                "string"
            ],
            "isRelic": True,
            "arraySpec": {
                "proxyHostId": "string"
            },
            "isPassthrough": True,
            "protectionDate": "2019-05-23T15:36:06.889Z",
            "snapshotCount": 0,
            "archivedSnapshotCount": 0,
            "snapshots": [
                {
                    "id": "string",
                    "date": "2019-05-23T15:36:06.889Z",
                    "expirationDate": "2019-05-23T15:36:06.889Z",
                    "sourceObjectType": "string",
                    "isOnDemandSnapshot": True,
                    "cloudState": 0,
                    "consistencyLevel": "string",
                    "indexState": 0,
                    "replicationLocationIds": [
                        "string"
                    ],
                    "archivalLocationIds": [
                        "string"
                    ],
                    "slaId": "string",
                    "slaName": "string",
                    "filesetName": "string",
                    "fileCount": 0
                }
            ],
            "localStorage": 0,
            "archiveStorage": 0,
            "preBackupScript": "string",
            "postBackupScript": "string",
            "backupScriptTimeout": 0,
            "backupScriptErrorHandling": "string"
        }

    mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
    mock_get.side_effect = [
        mock_get_v1_host(),
        mock_get_v1_fileset_template(),
        mock_get_v1_sla_domain(),
        mock_get_v1_fileset()]

    mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
    mock_post.return_value = mock_post_v1_fileset()

    mock_patch = mocker.patch('rubrik_cdm.Connect.patch', autospec=True, spec_set=True)
    mock_patch.return_value = mock_patch_v1_fileset()

    assert rubrik.assign_physical_host_fileset("hostname", "fileset_name", "Linux", "sla_name") == \
        (mock_post_v1_fileset(), mock_patch_v1_fileset())


#######
#######
#######
#######

# def test_aws_s3_cloudout_invalid_aws_region(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout(
#             "awsbucketname",
#             "archive_name",
#             "not_a_valid_aws_region",
#             "aws_region",
#             "aws_access_key",
#             "aws_secret_key")

#     error_message = error.value.args[0]

#     assert error_message == "The `aws_region` must be one of the following: ['ap-south-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1', 'ca-central-1', 'cn-north-1', 'cn-northwest-1', 'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'sa-east-1', 'us-gov-west-1', 'us-west-1', 'us-east-1', 'us-east-2', 'us-west-2']"


# def test_aws_s3_cloudout_invalid_storage_class(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout(
#             "awsbucketname",
#             "archive_name",
#             "ap-south-1",
#             "aws_access_key",
#             "aws_secret_key",
#             "kms_master_key_id",
#             "rsa_key",
#             "not_a_valid_storage_class")

#     error_message = error.value.args[0]

#     assert error_message == "The `storage_class` must be one of the following: ['standard', 'standard_ia', 'reduced_redundancy', 'onezone_ia']"


# @pytest.mark.parametrize('archive_name', ["_", "/", "*", "?", "%", ".", ":", "|", "<", ">"])
# def test_aws_s3_cloudout_invalid_aws_bucket_name(rubrik, archive_name):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout(archive_name)

#     error_message = error.value.args[0]

#     assert error_message == r"The `aws_bucket_name` may not contain any of the following characters: _\/*?%.:|<>"


# def test_aws_s3_cloudout_missing_aws_region(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout("awsbucketname")

#     error_message = error.value.args[0]

#     assert error_message == "`aws_region` has not been provided."


# def test_aws_s3_cloudout_missing_aws_access_key(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout("awsbucketname", "archive_name", "ap-south-1")

#     error_message = error.value.args[0]

#     assert error_message == "`aws_access_key` has not been provided."


# def test_aws_s3_cloudout_missing_aws_secret_key(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout("awsbucketname", "archive_name", "ap-south-1", "aws_access_key")

#     error_message = error.value.args[0]

#     assert error_message == "`aws_secret_key` has not been provided."


# def test_aws_s3_cloudout_missing_kms_and_rsa(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout(
#             "awsbucketname",
#             "archive_name",
#             "ap-south-1",
#             "aws_access_key",
#             "aws_secret_key")

#     error_message = error.value.args[0]

#     assert error_message == "You must populated either `kms_master_key_id` or `rsa_key`."


# def test_aws_s3_cloudout_both_kms_and_rsa_populated(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout(
#             "awsbucketname",
#             "archive_name",
#             "ap-south-1",
#             "aws_access_key",
#             "aws_secret_key",
#             "kms_master_key_id",
#             "rsa_key")

#     error_message = error.value.args[0]

#     assert error_message == "Both `kms_master_key_id` or `rsa_key` have been populated. You may only use one."


# def test_aws_s3_cloudout_idempotence(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "archive_name",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "string",
#                             "vNetId": "string",
#                             "securityGroupId": "string",
#                             "resourceGroupId": "string"
#                         },
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     assert rubrik.aws_s3_cloudout("awsbucketname", "archive_name", "ap-south-1", "aws_access_key", "aws_secret_key", None, "rsa_key") \
#         == "No change required. The 'archive_name' archival location is already configured on the Rubrik cluster."


# def test_aws_s3_cloudout_archive_name_already_exsits(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "archive_name",
#                         "accessKey": "string",
#                         "bucket": "string",
#                         "defaultRegion": "string",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "string",
#                             "vNetId": "string",
#                             "securityGroupId": "string",
#                             "resourceGroupId": "string"
#                         },
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudout(
#             "awsbucketname",
#             "archive_name",
#             "ap-south-1",
#             "aws_access_key",
#             "aws_secret_key",
#             None,
#             "rsa_key")

#     error_message = error.value.args[0]

#     assert error_message == "Archival location with name 'archive_name' already exists. Please enter a unique `archive_name`."


# def test_aws_s3_cloudout(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "string",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "string",
#                             "vNetId": "string",
#                             "securityGroupId": "string",
#                             "resourceGroupId": "string"
#                         },
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     def mock_post_internal_archive_object_store():
#         return {
#             "jobInstanceId": "string"
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
#     mock_post.return_value = mock_post_internal_archive_object_store()

#     assert rubrik.aws_s3_cloudout("awsbucketname", "archive_name", "ap-south-1", "aws_access_key", "aws_secret_key", None, "rsa_key") \
#         == mock_post_internal_archive_object_store()


# def test_update_aws_s3_cloudout_invalid_storage_class(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.update_aws_s3_cloudout("current_archive_name", storage_class="not_a_valid_storage_class")

#     error_message = error.value.args[0]

#     assert error_message == "The `storage_class` must be one of the following: ['standard', 'standard_ia', 'reduced_redundancy', 'onezone_ia']"


# def test_update_aws_s3_cloudout_current_archive_name_not_found(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "string",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "string",
#                             "vNetId": "string",
#                             "securityGroupId": "string",
#                             "resourceGroupId": "string"
#                         },
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.update_aws_s3_cloudout("current_archive_name")

#     error_message = error.value.args[0]

#     assert error_message == "No S3 archival location with name 'current_archive_name' exists."


# def test_update_aws_s3_cloudout(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "current_archive_name",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "string",
#                             "vNetId": "string",
#                             "securityGroupId": "string",
#                             "resourceGroupId": "string"
#                         },
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     def mock_patch_internal_archive_object_store():
#         return {
#             "jobInstanceId": "string"
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     mock_patch = mocker.patch('rubrik_cdm.Connect.patch', autospec=True, spec_set=True)
#     mock_patch.return_value = mock_patch_internal_archive_object_store()

#     assert rubrik.update_aws_s3_cloudout("current_archive_name", "new_archive_name", "aws_access_key", "aws_secret_key", "standard") \
#         == mock_patch_internal_archive_object_store()


# def test_aws_s3_cloudon_idempotence(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "archive_name",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "subnet_id",
#                             "vNetId": "vpc_id",
#                             "securityGroupId": "security_group_id",
#                         },
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     assert rubrik.aws_s3_cloudon("archive_name", "vpc_id", "subnet_id", "security_group_id") \
#         == "No change required. The 'archive_name' archival location is already configured for CloudOn."


# def test_aws_s3_cloudon_archive_name_not_found(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "string",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.aws_s3_cloudon("archive_name", "vpc_id", "subnet_id", "security_group_id")

#     error_message = error.value.args[0]

#     assert error_message == "The Rubrik cluster does not have an archive location named 'archive_name'."


# def test_aws_s3_cloudon(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "S3",
#                         "name": "archive_name",
#                         "accessKey": "aws_access_key",
#                         "bucket": "awsbucketname",
#                         "defaultRegion": "ap-south-1",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "storageClass": "STANDARD",
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     def mock_patch_internal_archive_object_store():
#         return {
#             "jobInstanceId": "string"
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     mock_patch = mocker.patch('rubrik_cdm.Connect.patch', autospec=True, spec_set=True)
#     mock_patch.return_value = mock_patch_internal_archive_object_store()

#     assert rubrik.aws_s3_cloudon("archive_name", "vpc_id", "subnet_id", "security_group_id") \
#         == mock_patch_internal_archive_object_store()


# @pytest.mark.parametrize('container', ["_", "/", "*", "?", "%", ".", ":", "|", "<", ">"])
# def test_azure_cloudout_invalid_container(rubrik, container):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.azure_cloudout(container, "azure_access_key", "storage_account_name", "rsa_key")

#     error_message = error.value.args[0]

#     assert error_message == r"The `container` may not contain any of the following characters: _\/*?%.:|<>"


# def test_azure_cloudout_invalid_instance_type(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.azure_cloudout("container", "azure_access_key", "storage_account_name",
#                               "rsa_key", instance_type="not_a_valid_instance_type")

#     error_message = error.value.args[0]

#     assert error_message == "The `instance_type` argument must be one of the following: ['default', 'china', 'germany', 'government']"


# def test_azure_cloudout_idempotence(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "Azure",
#                         "name": "archive_name",
#                         "accessKey": "storage_account_name",
#                         "bucket": "container",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     assert rubrik.azure_cloudout("container", "azure_access_key", "storage_account_name", "rsa_key", "archive_name") \
#         == "No change required. The 'archive_name' archival location is already configured on the Rubrik cluster."


# def test_azure_cloudout_invalid_archive_name(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "Azure",
#                         "name": "archive_name",
#                         "accessKey": "string",
#                         "bucket": "string",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.azure_cloudout("container", "azure_access_key", "storage_account_name", "rsa_key", "archive_name")

#     error_message = error.value.args[0]

#     assert error_message == "Archival location with name 'archive_name' already exists. Please enter a unique `name`."


# def test_azure_cloudout(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "Azure",
#                         "name": "string",
#                         "accessKey": "storage_account_name",
#                         "bucket": "container",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },

#                 }
#             ],
#             "total": 1
#         }

#     def mock_patch_internal_archive_object_store():
#         return {
#             "jobInstanceId": "string"
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
#     mock_post.return_value = mock_patch_internal_archive_object_store()

#     assert rubrik.azure_cloudout("container", "azure_access_key", "storage_account_name",
#                                  "rsa_key", "archive_name") == mock_patch_internal_archive_object_store()


# def test_azure_cloudon_invalid_region(rubrik):

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.azure_cloudon(
#             "archive_name",
#             "container",
#             "storage_account_name",
#             "application_id",
#             "application_key",
#             "tenant_id",
#             "not_a_valid_region",
#             "virtual_network_id",
#             "subnet_name",
#             "security_group_id")

#     error_message = error.value.args[0]

#     assert error_message == "The `region` must be one of the following: ['westus', 'westus2', 'centralus', 'eastus', 'eastus2', 'northcentralus', 'southcentralus', 'westcentralus', 'canadacentral', 'canadaeast', 'brazilsouth', 'northeurope', 'westeurope', 'uksouth', 'ukwest', 'eastasia', 'southeastasia', 'japaneast', 'japanwest', 'australiaeast', 'australiasoutheast', 'centralindia', 'southindia', 'westindia', 'koreacentral', 'koreasouth']"


# def test_azure_cloudon_idempotence(rubrik, mocker):

#     def mock_get_internal_archive_object_store():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "polarisManagedId": "string",
#                     "definition": {
#                         "objectStoreType": "Azure",
#                         "name": "archive_name",
#                         "accessKey": "storage_account_name",
#                         "bucket": "container",
#                         "isComputeEnabled": True,
#                         "isConsolidationEnabled": True,
#                         "defaultComputeNetworkConfig": {
#                             "subnetId": "subnet_name",
#                             "vNetId": "/subscriptions/89b90dec-a6e1-4q9e-bc12-23138bb3cee4/resourceGroups/PythonSDK/providers/Microsoft.Network/virtualNetworks/pythonsdk",
#                             "securityGroupId": "security_group_id",
#                             "resourceGroupId": "string"
#                         },
#                         "azureComputeSummary": {
#                             "tenantId": "tenant_id",
#                             "subscriptionId": "89b90dec-a6e1-4q9e-bc12-23138bb3cee4",
#                             "clientId": "application_id",
#                             "region": "westus",
#                             "generalPurposeStorageAccountName": "storage_account_name",
#                             "containerName": "container",
#                             "environment": "AZURE"
#                         },
#                         "encryptionType": "RSA_KEY_ENCRYPTION"
#                     },


#                 }
#             ],
#             "total": 1
#         }

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_archive_object_store()

#     assert rubrik.azure_cloudon(
#         "archive_name",
#         "container",
#         "storage_account_name",
#         "application_id",
#         "application_key",
#         "tenant_id",
#         "westus",
#         "/subscriptions/89b90dec-a6e1-4q9e-bc12-23138bb3cee4/resourceGroups/PythonSDK/providers/Microsoft.Network/virtualNetworks/pythonsdk",
#         "subnet_name",
#         "security_group_id") == "No change required. The 'archive_name' archival location is already configured for CloudOn."


# def test_update_aws_native_account_minimum_installed_cdm_version_not_met(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.0.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(CDMVersionException) as error:
#         rubrik.update_aws_native_account("aws_account_name", {})


# def test_update_aws_native_account_invalid_config(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidTypeException) as error:
#         rubrik.update_aws_native_account("aws_account_name", "not_a_dictionary")

#     error_message = error.value.args[0]

#     assert error_message == "The 'config' argument must be a dictionary."


# def test_update_aws_native_account(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     def mock_get_internal_aws_account_name():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "name": "aws_account_name",
#                     "primaryClusterId": "string",
#                     "status": "Connected"
#                 }
#             ],
#             "total": 1
#         }

#     def mock_patch_internal_aws_account_id():
#         return {
#             "name": "string",
#             "accessKey": "string",
#             "regions": [
#                 "string"
#             ],
#             "regionalBoltNetworkConfigs": [
#                 {
#                     "region": "string",
#                     "vNetId": "string",
#                     "subnetId": "string",
#                     "securityGroupId": "string"
#                 }
#             ],
#             "disasterRecoveryArchivalLocationId": "string",
#             "id": "string",
#             "configuredSlaDomainId": "string",
#             "configuredSlaDomainName": "string",
#             "primaryClusterId": "string"
#         }

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_aws_account_name()

#     mock_patch = mocker.patch('rubrik_cdm.Connect.patch', autospec=True, spec_set=True)
#     mock_patch.return_value = mock_patch_internal_aws_account_id()

#     assert rubrik.update_aws_native_account("aws_account_name", {}) == \
#         mock_patch_internal_aws_account_id()


# def test_add_aws_native_account_minimum_installed_cdm_version_not_met(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.0.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(CDMVersionException) as error:
#         rubrik.add_aws_native_account(
#             "aws_account_name",
#             "aws_access_key",
#             "aws_secret_key",
#             ["not_a_valid_region"],
#             {})


# def test_add_aws_native_account_missing_aws_region(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.add_aws_native_account("aws_account_name")

#     error_message = error.value.args[0]

#     assert error_message == "`aws_region` has not been provided."


# def test_add_aws_native_account_missing_aws_access_key(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.add_aws_native_account("aws_account_name", aws_regions=["ap-south-1"])

#     error_message = error.value.args[0]

#     assert error_message == "`aws_access_key` has not been provided."


# def test_add_aws_native_account_missing_aws_secret_key(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.add_aws_native_account("aws_account_name", "aws_access_key", aws_regions=["ap-south-1"])

#     error_message = error.value.args[0]

#     assert error_message == "`aws_secret_key` has not been provided."


# def test_add_aws_native_account_invalid_aws_regionss(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.add_aws_native_account(
#             "aws_account_name",
#             "aws_access_key",
#             "aws_secret_key",
#             ["not_a_valid_region"],
#             {})

#     error_message = error.value.args[0]

#     assert error_message == "The list `aws_regions` may only contain the following values: ['ap-south-1', 'ap-northeast-3', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1', 'ca-central-1', 'cn-north-1', 'cn-northwest-1', 'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'us-west-1', 'us-east-1', 'us-east-2', 'us-west-2']"


# def test_add_aws_native_account_invalid_regional_bolt_network_configs_list(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidTypeException) as error:
#         rubrik.add_aws_native_account(
#             "aws_account_name",
#             "aws_access_key",
#             "aws_secret_key",
#             ["ap-south-1"],
#             "not_a_valid_regional_bolt_config")

#     error_message = error.value.args[0]

#     assert error_message == "`regional_bolt_network_configs` must be a list if defined."


# def test_add_aws_native_account_invalid_regional_bolt_network_configs_list_dict(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidTypeException) as error:
#         rubrik.add_aws_native_account(
#             "aws_account_name",
#             "aws_access_key",
#             "aws_secret_key",
#             ["ap-south-1"],
#             ["not_a_valid_regional_bolt_config"])

#     error_message = error.value.args[0]

#     assert error_message == "The `regional_bolt_network_configs` list can only contain dicts."


# def test_add_aws_native_account_invalid_regional_bolt_network_configs_list_dict_value(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.add_aws_native_account(
#             "aws_account_name",
#             "aws_access_key",
#             "aws_secret_key",
#             ["ap-south-1"],
#             [{"not_a_valid_key": "string"}])

#     error_message = error.value.args[0]

#     assert error_message == "Each `regional_bolt_network_config` dict must contain the following keys: 'region', 'vNetId', 'subnetId', 'securityGroupId'."


# def test_add_aws_native_account_invalid_aws_account_name(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     def mock_get_internal_aws_account_name():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "name": "aws_account_name",
#                     "primaryClusterId": "string",
#                     "status": "Connected"
#                 }
#             ],
#             "total": 1
#         }

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.return_value = mock_get_internal_aws_account_name()

#     with pytest.raises(InvalidParameterException) as error:
#         rubrik.add_aws_native_account(
#             "aws_account_name",
#             "aws_access_key",
#             "aws_secret_key",
#             ["ap-south-1"],
#             [{"region": "string", "vNetId": "string", "subnetId": "string", "securityGroupId": "string"}])

#     error_message = error.value.args[0]

#     assert error_message == "Cloud native source with name 'aws_account_name' already exists. Please enter a unique `aws_account_name`."


# def test_add_aws_native_account_idempotence(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     def mock_get_internal_aws_account_name():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "name": "string",
#                     "primaryClusterId": "string",
#                     "status": "Connected"
#                 }
#             ],
#             "total": 1
#         }

#     def mock_get_internal_aws_account_id():
#         return {
#             "name": "string",
#             "accessKey": "aws_access_key",
#             "regions": [
#                 "string"
#             ],
#             "regionalBoltNetworkConfigs": [
#                 {
#                     "region": "string",
#                     "vNetId": "string",
#                     "subnetId": "string",
#                     "securityGroupId": "string"
#                 }
#             ],
#             "disasterRecoveryArchivalLocationId": "string",
#             "id": "string",
#             "configuredSlaDomainId": "string",
#             "configuredSlaDomainName": "string",
#             "primaryClusterId": "string"
#         }

#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.side_effect = [mock_get_internal_aws_account_name(), mock_get_internal_aws_account_id()]

#     assert rubrik.add_aws_native_account("aws_account_name",
#                                          "aws_access_key",
#                                          "aws_secret_key",
#                                          ["ap-south-1"],
#                                          [{"region": "string",
#                                            "vNetId": "string",
#                                            "subnetId": "string",
#                                            "securityGroupId": "string"}]) == "No change required. Cloud native source with access key 'aws_access_key' is already configured on the Rubrik cluster."


# def test_add_aws_native_account(rubrik, mocker):

#     def mock_self_cluster_version():
#         return "4.2.1-1280"

#     def mock_get_internal_aws_account_name():
#         return {
#             "hasMore": True,
#             "data": [
#                 {
#                     "id": "string",
#                     "name": "string",
#                     "primaryClusterId": "string",
#                     "status": "Connected"
#                 }
#             ],
#             "total": 1
#         }

#     def mock_get_internal_aws_account_id():
#         return {
#             "name": "string",
#             "accessKey": "string",
#             "regions": [
#                 "string"
#             ],
#             "regionalBoltNetworkConfigs": [
#                 {
#                     "region": "string",
#                     "vNetId": "string",
#                     "subnetId": "string",
#                     "securityGroupId": "string"
#                 }
#             ],
#             "disasterRecoveryArchivalLocationId": "string",
#             "id": "string",
#             "configuredSlaDomainId": "string",
#             "configuredSlaDomainName": "string",
#             "primaryClusterId": "string"
#         }

#     def mock_post_internal_aws_account():
#         return {
#             "id": "string",
#             "status": "string",
#             "progress": 0,
#             "startTime": "2019-05-03T16:15:38.827Z",
#             "endTime": "2019-05-03T16:15:38.827Z",
#             "nodeId": "string",
#             "error": {
#                 "message": "string"
#             },
#             "links": [
#                 {
#                     "href": "string",
#                     "rel": "string"
#                 }
#             ]
#         }
#     mock_cluster_version = mocker.patch('rubrik_cdm.Connect.cluster_version', autospec=True, spec_set=True)
#     mock_cluster_version.return_value = mock_self_cluster_version()

#     mock_get = mocker.patch('rubrik_cdm.Connect.get', autospec=True, spec_set=True)
#     mock_get.side_effect = [mock_get_internal_aws_account_name(), mock_get_internal_aws_account_id()]

#     mock_post = mocker.patch('rubrik_cdm.Connect.post', autospec=True, spec_set=True)
#     mock_post.return_value = mock_post_internal_aws_account()

#     assert rubrik.add_aws_native_account("aws_account_name",
#                                          "aws_access_key",
#                                          "aws_secret_key",
#                                          ["ap-south-1"],
#                                          [{"region": "string",
#                                            "vNetId": "string",
#                                            "subnetId": "string",
#                                            "securityGroupId": "string"}]) == mock_post_internal_aws_account()



# Copyright 2018 Rubrik, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.

"""
This module contains the Rubrik SDK Cluster class.
"""

from .api import Api
from .exceptions import InvalidParameterException, CDMVersionException, InvalidTypeException


class Cluster(Api):
    """This class contains methods related to the managment of the Rubrik cluster itself.
    """

    def cluster_version(self, timeout=15):
        """Retrieves the software version of the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- The version of CDM installed on the Rubrik cluster.
        """

        self.log('cluster_version: Getting the software version of the Rubrik cluster.')

        return self.get('v1', '/cluster/me/version', timeout=timeout)['version']

    def minimum_installed_cdm_version(self, cluster_version, timeout=15):
        """Determine if the Rubrik cluster is running the provided CDM `cluster_version` or later. If the cluster is running an earlier release
        of CDM, `False` is returned. If the cluster is running the provided `cluster_version`, or a later release, `True` is returned.

        Arguments:
            cluster_version {float} -- The minimum required version of Rubrik CDM you wish ensure is running.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})
        """

        if float(self.cluster_version(timeout)[:3]) < float(cluster_version):
            return False

        return True

    def cluster_node_ip(self, timeout=15):
        """Retrive the IP Address for each node in the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            list -- A list that contains the IP Address for each node in the Rubrik cluster.
        """

        self.log('cluster_node_ip: Generating a list of all Cluster Node IPs.')
        api_request = self.get('internal', '/cluster/me/node', timeout=timeout)

        node_ip_list = []

        for node in api_request['data']:
            node_ip_list.append(node["ipAddress"])

        return node_ip_list

    def cluster_node_name(self, timeout=15):
        """Retrive the name of each node in the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            list -- A list that contains the name of each node in the Rubrik cluster.
        """

        self.log('cluster_node_ip: Generating a list of all Cluster')
        api_request = self.get('internal', '/cluster/me/node', timeout=timeout)

        node_ip_name = []

        for node in api_request['data']:
            node_ip_name.append(node["id"])

        return node_ip_name

    def end_user_authorization(self, object_name, end_user, object_type='vmware', timeout=15):
        """Grant an End User authorization to the provided object.

        Arguments:
            object_name {str} -- The name of the object you wish to grant the `end_user` authorization to.
            end_user {str} -- The name of the end user you wish to grant authorization to.

        Keyword Arguments:
            object_type {str} -- The Rubrik object type you wish to backup. (default: {vmware}) (choices: {vmware})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The End User "`end_user`" is already authorized to interact with the "`object_name`" VM.
            dict -- The full API response from `POST /internal/authorization/role/end_user`.
        """

        valid_object_type = ['vmware']

        if object_type not in valid_object_type:
            raise InvalidParameterException("The end_user_authorization() object_type argument must be one of the following: {}.".format(
                valid_object_type))

        self.log("end_user_authorization: Searching the Rubrik cluster for the vSphere VM '{}'.".format(
            object_name))
        vm_id = self.object_id(object_name, object_type, timeout=timeout)

        self.log(
            "end_user_authorization: Searching the Rubrik cluster for the End User '{}'.".format(end_user))
        user = self.get(
            'internal', '/user?username={}'.format(end_user), timeout=timeout)

        if not user:
            raise InvalidParameterException(
                'The Rubrik cluster does not contain a End User account named "{}".'.format(end_user))
        else:
            user_id = user[0]['id']

        self.log(
            "end_user_authorization: Searching the Rubrik cluster for the End User '{}' authorizations.".format(end_user))
        user_authorization = self.get(
            'internal', '/authorization/role/end_user?principals={}'.format(user_id), timeout=timeout)

        authorized_objects = user_authorization['data'][0]['privileges']['restore']

        if vm_id in authorized_objects:
            return 'No change required. The End User "{}" is already authorized to interact with the "{}" VM.'.format(
                end_user, object_name)
        else:
            config = {}
            config['principals'] = [user_id]
            config['privileges'] = {"restore": [vm_id]}

            return self.post(
                'internal',
                '/authorization/role/end_user',
                config,
                timeout=timeout)

    def add_vcenter(self, vcenter_ip, vcenter_username, vcenter_password, vm_linking=True, ca_certificate=None, timeout=30):  # pylint: ignore
        """Add a new vCenter to the Rubrik cluster.

        Arguments:
            vcenter_ip {str} -- The IP address or FQDN of the vCenter you wish to add.
            vcenter_username {str} -- The vCenter username used for authentication.
            vcenter_password {str} -- The vCenter password used for authentication.

        Keyword Arguments:
            vm_linking {bool} -- Automatically link discovered virtual machines (i.e VM Linking). (default: {True})
            ca_certificate {str} -- CA certificiate used to perform TLS certificate validation (default: {None})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {30})

        Returns:
            str -- No change required. The vCenter '`vcenter_ip`' has already been added to the Rubrik cluster.
            tuple -- The full API response for `POST /v1/vmware/vcenter` and the job status URL which can be used to monitor progress of the adding the vCenter to the Rubrik cluster. (api_response, job_status_url)
        """

        self.log("add_vcenter: Searching the Rubrik cluster for the vCenter '{}'.".format(vcenter_ip))
        current_vcenter = self.get("v1", "/vmware/vcenter?primary_cluster_id=local", timeout=timeout)

        for vcenter in current_vcenter["data"]:
            if vcenter["hostname"] == vcenter_ip:
                return "No change required. The vCenter '{}' has already been added to the Rubrik cluster.".format(
                    vcenter_ip)

        config = {}
        config["hostname"] = vcenter_ip
        config["username"] = vcenter_username
        config["password"] = vcenter_password
        if vm_linking:
            config["conflictResolutionAuthz"] = "AllowAutoConflictResolution"
        elif vm_linking is False:
            config["conflictResolutionAuthz"] = "NoConflictResolution"
        if ca_certificate is not None:
            config["caCerts"] = ca_certificate

        self.log("add_vcenter: Adding vCenter '{}' to the Rubrik cluster.".format(vcenter_ip))
        add_vcenter = self.post("v1", "/vmware/vcenter", config, timeout)

        return add_vcenter, add_vcenter['links'][0]['href']

    def configure_timezone(self, timezone, timeout=15):
        """Configure the Rubrik cluster timezone.

        Arguments:
            timezone {str} -- The timezone you wish the Rubrik cluster to use. (choices: {America/Anchorage, America/Araguaina, America/Barbados, America/Chicago, America/Denver, America/Los_Angeles, America/Mexico_City, America/New_York, America/Noronha, America/Phoenix, America/Toronto, America/Vancouver, Asia/Bangkok, Asia/Dhaka, Asia/Dubai, Asia/Hong_Kong, Asia/Karachi, Asia/Kathmandu, Asia/Kolkata, Asia/Magadan, Asia/Singapore, Asia/Tokyo, Atlantic/Cape_Verde, Australia/Perth, Australia/Sydney, Europe/Amsterdam, Europe/Athens, Europe/London, Europe/Moscow, Pacific/Auckland, Pacific/Honolulu, Pacific/Midway, UTC})

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster is already configured with '`timezone`' as it's timezone.
            dict -- The full API response for `PATCH /v1//cluster/me'`
        """

        valid_timezones = [
            'America/Anchorage',
            'America/Araguaina',
            'America/Barbados',
            'America/Chicago',
            'America/Denver',
            'America/Los_Angeles',
            'America/Mexico_City',
            'America/New_York',
            'America/Noronha',
            'America/Phoenix',
            'America/Toronto',
            'America/Vancouver',
            'Asia/Bangkok',
            'Asia/Dhaka',
            'Asia/Dubai',
            'Asia/Hong_Kong',
            'Asia/Karachi',
            'Asia/Kathmandu',
            'Asia/Kolkata',
            'Asia/Magadan',
            'Asia/Singapore',
            'Asia/Tokyo',
            'Atlantic/Cape_Verde',
            'Australia/Perth',
            'Australia/Sydney',
            'Europe/Amsterdam',
            'Europe/Athens',
            'Europe/London',
            'Europe/Moscow',
            'Pacific/Auckland',
            'Pacific/Honolulu',
            'Pacific/Midway',
            'UTC']

        if timezone not in valid_timezones:
            raise InvalidParameterException("The timezone argument must be one of the following: {}.".format(
                valid_timezones))

        self.log("cluster_timezone: Determing the current cluster timezone")
        cluster_summary = self.get("v1", "/cluster/me", timeout=timeout)

        if cluster_summary["timezone"]["timezone"] == timezone:
            return "No change required. The Rubrik cluster is already configured with '{}' as it's timezone.".format(
                timezone)

        config = {}
        config["timezone"] = {}
        config["timezone"]["timezone"] = timezone

        self.log("cluster_timezone: Configuring the Rubrik cluster timezone")
        return self.patch("v1", "/cluster/me", config, timeout)

    def configure_ntp(self, ntp_server, timeout=15):
        """Configure connection information for the NTP servers used by the Rubrik cluster for time synchronization.

        Arguments:
            ntp_server {list} -- A list of the NTP server(s) you wish to configure the Rubrik cluster to use.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The NTP server(s) `ntp_server` has already been added to the Rubrik cluster.
            dict -- {'status_code': 204}
        """

        if isinstance(ntp_server, list) is False:
            raise InvalidTypeException("The 'ntp_server' argument must be a list object.")

        self.log("cluster_ntp: Determing the current cluster NTP settings")
        cluster_ntp = self.get("internal", "/cluster/me/ntp_server", timeout=timeout)

        if sorted(cluster_ntp["data"]) == sorted(ntp_server):
            return "No change required. The NTP server(s) {} has already been added to the Rubrik cluster.".format(
                ntp_server)

        self.log(
            "cluster_ntp: Adding the NTP server(s) '{}' to the Rubrik cluster.".format(ntp_server))
        return self.post("internal", "/cluster/me/ntp_server", ntp_server, timeout)

    def configure_syslog(self, syslog_ip, protocol, port=514, timeout=15):
        """Configure the Rubrik cluster syslog settings.

        Arguments:
            syslog_ip {str} -- The IP address or hostname of the syslog server you wish to add to the Rubrik cluster.
            protocol {str} -- The protocol to use when making the connection to the syslog server. (choices: {TCP, UDP})

        Keyword Arguments:
            port {int} -- The port to use when making the connection to the syslog server. (default: {514})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster is already configured to use the syslog server '`syslog_hostname`' on port '`port`' using the '`protocol`' protocol.
            dict -- The full API response for `POST /internal/syslog'`
        """

        valid_protocols = ["TCP", "UDP"]

        if protocol not in valid_protocols:
            raise InvalidParameterException("The protocol argument must be one of the following: {}.".format(
                valid_protocols))

        self.log("cluster_syslog: Getting the current cluster syslog settings")
        syslog = self.get("internal", "/syslog", timeout=timeout)

        config = {}
        config["hostname"] = syslog_ip
        config["protocol"] = protocol
        # convert to an int in case the user provides a string
        config["port"] = int(port)

        if syslog["total"] != 0:

            current_syslog_config = syslog["data"][0]
            # remove the id key from the syslog config for comparison
            del current_syslog_config["id"]

            if current_syslog_config == config:
                return "No change required. The Rubrik cluster is already configured to use the syslog server '{}' on port '{}' using the '{}' protocol.".format(
                    syslog_ip, port, protocol)

            self.log("cluster_syslog: Clearing the existing syslog configuration.")
            self.delete("internal", "/syslog/1")

        self.log("cluster_syslog: Configuring the syslog settings.")
        return self.post("internal", "/syslog", config, timeout)

    def configure_vlan(self, vlan, netmask, ips, timeout=15):
        """Configure VLANs on the Rubrik cluster.

        Arguments:
            vlan {int} -- The VLAN ID you wish to configure.
            netmask {str} -- The netmask of the VLAN ID you wish to configure.
            ips {list or dict} -- The `ips` argument can either be a list or a dictionary. The list should contain an IP, in the relevant VLAN, for each node in the cluster. These IPs will be sorted, from lowest to highest, and then automatically associated with a node name based on alphabetical order. If you would like more finite control over the assignment you can use a dict with `node_name:ip` as it's key pairs.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster is already configured with the provided VLAN information.
            dict -- The full API response for `POST /internal/cluster/me/vlan'`
        """

        if isinstance(ips, list):
            self.log("cluster_vlan: Generating a list of all Cluster Node IPs.")
            node_names = self.cluster_node_name()

            if len(node_names) != len(ips):
                raise InvalidParameterException("The Rubrik cluster has {} nodes but you provided {} IP addresses. There must be a 1 to 1 relationship between nodes and IPs.".format(
                    str(len(node_names)), str(len(ips))))

            node_names = sorted(node_names)
            interfaces = sorted(ips)

            node_ip_combined = {}
            for i in range(0, len(node_names)):
                node_ip_combined[node_names[i]] = interfaces[i]
        elif isinstance(ips, dict):
            node_ip_combined = ips
        else:
            raise InvalidParameterException(
                "The interfaces argument must be either a list of IPs or a dictionary with node_name:ip as the key, value pairs.")

        self.log("cluster_vlan: Getting the current VLAN configurations.")
        current_vlans = self.get("internal", "/cluster/me/vlan", timeout=timeout)
        if current_vlans["total"] != 0:
            current_vlans = current_vlans["data"][0]

        interface_node_ip = [{'node': key, 'ip': val}
                             for key, val in node_ip_combined.items()]

        config = {}
        config["vlan"] = int(vlan)
        config["netmask"] = netmask
        config["interfaces"] = interface_node_ip

        if current_vlans == config:
            return "No change required. The Rubrik cluster is already configured with the provided VLAN information."

        self.log("cluster_vlan: Configuring the VLANs.")
        return self.post("internal", "/cluster/me/vlan", config, timeout)

    def configure_dns_servers(self, server_ip, timeout=15):
        """Configure the DNS Servers on the Rubrik cluster.

        Arguments:
            server_ip {list} -- The DNS Server IPs you wish to add to the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster is already configured with the provided DNS servers.
            dict -- The full API response for `POST /internal/cluster/me/dns_nameserver'`
        """

        if isinstance(server_ip, list) is False:
            raise InvalidTypeException("The 'server_ip' argument must be a list")

        self.log("cluster_dns_servers: Generating a list of DNS servers configured on the Rubrik cluster.")
        current_dns_servers = self.get("internal", "/cluster/me/dns_nameserver", timeout=timeout)

        if sorted(current_dns_servers) == sorted(server_ip):
            return "No change required. The Rubrik cluster is already configured with the provided DNS servers."

        return self.post("internal", "/cluster/me/dns_nameserver", server_ip, timeout)

    def configure_search_domain(self, search_domain, timeout=15):
        """Configure the DNS search domains on the Rubrik cluster.

        Arguments:
            search_domain {list} -- The DNS search domains you wish to add to the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster is already configured with the provided DNS search domains.
            dict -- The full API response for `POST /internal/cluster/me/dns_search_domain'`
        """

        if isinstance(search_domain, list) is False:
            raise InvalidTypeException("The 'server_ip' argument must be a list")

        self.log("cluster_dns_servers: Generating a list of DNS servers configured on the Rubrik cluster.")
        current_dns_search_domains = self.get("internal", "/cluster/me/dns_search_domain", timeout=timeout)

        if sorted(current_dns_search_domains) == sorted(search_domain):
            return "No change required. The Rubrik cluster is already configured with the provided DNS Search Domains."

        return self.post("internal", "/cluster/me/dns_search_domain", search_domain, timeout)

    def configure_smtp_settings(self, hostname, port, from_email, smtp_username, smtp_password, encryption="NONE", timeout=15):  # pylint: ignore
        """The Rubrik cluster uses email to send all notifications to local Rubrik cluster user accounts that have the Admin role. To do this the Rubrik cluster transfers the email messages to an SMTP server for delivery.
        This function will configure the Rubrik cluster with account information for the SMTP server to permit the Rubrik cluster to use the SMTP server for sending outgoing email.

        Arguments:
            hostname {str} -- Hostname of the SMTP server.
            port {int} -- Incoming port on the SMTP server. Normally port 25, port 465, or port 587, depending upon the type of encryption used.
            from_email {str} --  The email address assigned to the account on the SMTP server
            smtp_username {str} -- The username assigned to the account on the SMTP server
            smtp_password {str} -- The password associated with the username

        Keyword Arguments:
            encryption {str} --  The encryption protocol that the SMTP server requires for incoming SMTP connections (default: {"NONE"}) (choices: {NONE, SSL, STARTTLS})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster is already configured with the provided SMTP settings.
            dict -- The full API response for `POST /internal/smtp_instance'`
            dict -- The full API response for `PATCH /internal/smtp_instance/{id}'`
        """

        valid_encryption = ['SSL', 'STARTTLS', 'NONE']

        if encryption not in valid_encryption:
            raise InvalidParameterException("cluster_smtp_settings() encryption argument must be one of the following: {}.".format(
                valid_encryption))

        self.log("cluster_smtp_settings: Determing the current SMTP settings on the Rubrik cluster.")
        current_smtp_settings = self.get("internal", "/smtp_instance", timeout=timeout)

        config = {}
        config["smtpHostname"] = hostname
        config["smtpPort"] = int(port)
        config["smtpSecurity"] = encryption
        config["smtpUsername"] = smtp_username
        config["fromEmailId"] = from_email

        if current_smtp_settings["total"] == 0:
            config["smtpPassword"] = smtp_password
            self.log("cluster_smtp_settings: Configuring the SMTP settings.")
            return self.post("internal", "/smtp_instance", config, timeout)
        else:
            current_smtp_settings = current_smtp_settings["data"][0]
            # Save the SMTP ID in case of PATCH and then delete for comparison
            smtp_id = current_smtp_settings["id"]
            del current_smtp_settings["id"]

            if current_smtp_settings == config:
                return "No change required. The Rubrik cluster is already configured with the provided SMTP settings."

            self.log("cluster_smtp_settings: Updating the SMTP settings.")
            return self.patch("internal", "/smtp_instance/{}".format(smtp_id), config, timeout)

    def refresh_vcenter(self, vcenter_ip, wait_for_completion=True, timeout=15):
        """Refresh the metadata for the specified vCenter Server.

        Arguments:
            vcenter_ip {str} -- The IP address or FQDN of the vCenter you wish to refesh.


        Keyword Arguments:
            wait_for_completion {bool} -- Flag to determine if the function should wait for the refresh to complete before completing. (default: {True})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            dict -- When wait_for_completion is False, the full API response for `POST /v1/vmware/vcenter/{id}/refresh`
            dict -- When wait_for_completion is True, the full API response of the job status
        """

        self.log("refresh_vcenter: Searching the Rubrik cluster for the provided vCenter Server.")
        vcenter_id = self.object_id(vcenter_ip, "vcenter", timeout=timeout)

        self.log("refresh_vcenter: Refresh vCenter.")
        api_request = self.post("v1", "/vmware/vcenter/{}/refresh".format(vcenter_id), timeout)

        if wait_for_completion:
            return self.job_status(api_request["links"][0]["href"])

        return api_request

    def create_user(self, username, password, first_name=None, last_name=None, email_address=None, contact_number=None, timeout=15):  # pylint: ignore
        """Create a new user on the Rubrik cluster

        Arguments:
            username {str} -- The username for the user you wish to create.
            password {str} -- The password for the user you wish to create.

        Keyword Arguments:
            first_name {str} -- The first name of the user you wish to create. (default: {None})
            last_name {str} -- The last name of the user you wish to create. (default: {None})
            email_address {str} -- The email address of the user you wish to create. (default: {None})
            contact_number {str} -- The contact number of the user you wish to create. (default: {None})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The user '`username`' already exists on the Rubrik cluster
            dict -- The full API response from `POST /internal/user`.
        """

        self.log("create_user: Searching for the current users on the Rubrik cluster")
        current_users = self.get("internal", "/user?username={}".format(username), timeout=timeout)
        if len(current_users) > 0:
            return "No change required. The user '{}' already exists on the Rubrik cluster.".format(username)

        config = {}
        config["username"] = username
        config["password"] = password
        if first_name is not None:
            config["firstName"] = first_name
        if last_name is not None:
            config["lastName"] = last_name
        if email_address is not None:
            config["emailAddress"] = email_address
        if contact_number is not None:
            config["contactNumber"] = contact_number

        self.log("create_user: Creating the new user account.")
        return self.post("internal", "/user", config, timeout)

    def read_only_authorization(self, username, timeout=15):
        """Grant read-only access to a specific user.

        Arguments:
            username {str} -- The username you wish to grant read-only access to.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The user '`username`' already has read-only permissions.
            dict -- The full API response from `POST /internal/authorization/role/read_only_admin`.
        """

        if self.minimum_installed_cdm_version(5.0) is False:
            raise CDMVersionException(5.0)

        self.log("read_only_authorization: Searching for the current users on the Rubrik cluster")
        current_users = self.get("internal", "/user?username={}".format(username), timeout=timeout)
        if len(current_users) < 1:
            raise InvalidParameterException(
                "The user '{}' does not exsit on the Rubrik cluster.".format(username))

        self.log("read_only_authorization: Checking the current authorizations for user '{}'".format(username))
        current_authorizations = self.get(
            "internal", "/authorization/role/read_only_admin?principals={}".format(current_users[0]["id"]), timeout=timeout)

        try:
            if current_authorizations["data"][0]["privileges"]["basic"][0] == "Global:::All":
                return "No change required. The user '{}' already has read-only permissions.".format(username)
        except BaseException:
            pass

        config = {}
        config["principals"] = [current_users[0]["id"]]
        config["privileges"] = {}
        config["privileges"]["basic"] = ["Global:::All"]

        self.log("read_only_authorization: Granting read-only privilages to user '{}'.".format(username))
        return self.post("internal", "/authorization/role/read_only_admin", config, timeout)
